(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('classnames'), require('polished'), require('react-style-system'), require('nanoid'), require('react-dom')) :
    typeof define === 'function' && define.amd ? define(['exports', 'react', 'classnames', 'polished', 'react-style-system', 'nanoid', 'react-dom'], factory) :
    (global = global || self, factory(global.HackerUI = {}, global.React, global.classNames, global.polished, global.ReactStyleSystem, global.nanoId, global.ReactDOM));
}(this, (function (exports, React, classNames, polished, reactStyleSystem, nanoId, reactDom) { 'use strict';

    var React__default = 'default' in React ? React['default'] : React;
    classNames = classNames && Object.prototype.hasOwnProperty.call(classNames, 'default') ? classNames['default'] : classNames;
    nanoId = nanoId && Object.prototype.hasOwnProperty.call(nanoId, 'default') ? nanoId['default'] : nanoId;

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    function __rest(s, e) {
        var t = {};
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
            t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }

    function __awaiter(thisArg, _arguments, P, generator) {
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }

    const useStyles = reactStyleSystem.createStyles(({ css, theme, color }) => ({
        // button base styles
        root: css `
    ${theme.fonts.button}

    display: inline-flex;
    justify-content: center;
    align-items: center;

    appearance: none;
    outline: none;
    background: transparent;
    border: none;
    text-decoration: none;
    cursor: pointer;

    &:disabled {
      cursor: not-allowed;
    }
  `,
        // shapes
        standard: css `
    min-width: ${theme.block(1)};

    & > *:not(:last-child) {
      margin-right: ${theme.space(1)};
    }
  `,
        icon: css `
    border-radius: 99999px;
    & > svg {
      width: ${theme.space(1)};
      height: ${theme.space(1)};
    }
  `,
        // variants
        outlined: css `
    border: 1px solid ${color.readable};
    color: ${color.readable};

    transition: background-color ${theme.durations.standard},
      border ${theme.durations.standard};

    &:focus {
      background-color: ${polished.transparentize(0.92, color.decorative)};
    }
    &:hover {
      background-color: ${polished.transparentize(0.9, color.decorative)};
    }
    &:active {
      background-color: ${polished.transparentize(0.8, color.decorative)};
    }
    &:disabled {
      color: ${polished.transparentize(0.4, color.readable)};
      background-color: transparent;
    }
  `,
        filled: css `
    background-color: ${color.decorative};
    color: ${polished.readableColor(color.decorative)};
    border: 2px solid transparent;
    transition: background-color ${theme.durations.standard},
      border ${theme.durations.standard};

    &:focus {
      background-color: ${polished.transparentize(0.18, color.decorative)};
      border: 2px solid ${polished.transparentize(0.7, '#fff')};
    }
    &:hover {
      background-color: ${polished.transparentize(0.23, color.decorative)};
    }
    &:active {
      background-color: ${polished.transparentize(0.3, color.decorative)};
      border: 2px solid ${polished.transparentize(0.5, '#fff')};
    }
    &:disabled {
      background-color: ${polished.transparentize(0.5, color.decorative)};
      border: 2px solid transparent;
      color: ${polished.transparentize(0.2, polished.readableColor(color.decorative))};
    }
  `,
        ghost: css `
    color: ${color.readable};
    border: 2px solid transparent;
    transition: background-color ${theme.durations.standard},
      border ${theme.durations.standard};

    &:focus {
      background-color: ${polished.transparentize(0.92, color.decorative)};
      border: 2px solid ${polished.transparentize(0.8, polished.readableColor(color.decorative))};
    }
    &:hover {
      background-color: ${polished.transparentize(0.9, color.decorative)};
    }
    &:active {
      background-color: ${polished.transparentize(0.8, color.decorative)};
    }
    &:disabled {
      color: ${polished.transparentize(0.4, color.readable)};
      background-color: transparent;
    }
  `,
        // sizes
        standardSmall: css `
    padding: ${theme.space(0.5)} ${theme.space(0.75)};
    min-width: ${theme.block(1)};
  `,
        standardStandard: css `
    padding: ${theme.space(0.75)} ${theme.space(1)};
    min-width: ${theme.block(1)};
  `,
        standardLarge: css `
    padding: ${theme.space(1)} ${theme.space(1.25)};
    min-width: ${theme.block(1.5)};
  `,
        iconSmall: css `
    width: ${theme.space(2)};
    height: ${theme.space(2)};
  `,
        iconStandard: css `
    width: ${theme.space(2.5)};
    height: ${theme.space(2.5)};
  `,
        iconLarge: css `
    width: ${theme.space(3)};
    height: ${theme.space(3)};
  `,
    }));
    const Button = React.forwardRef((props, ref) => {
        const _a = useStyles(props, props.component || 'button'), { Root, styles, component, variant = 'ghost', size = 'standard', shape = 'standard' } = _a, restOfProps = __rest(_a, ["Root", "styles", "component", "variant", "size", "shape"]);
        return (React__default.createElement(Root, Object.assign({ className: classNames('hui-button', {
                [styles.icon]: shape === 'icon',
                [styles.standard]: shape === 'standard',
                [styles.outlined]: variant === 'outlined',
                [styles.ghost]: variant === 'ghost',
                [styles.filled]: variant === 'filled',
                [styles.iconSmall]: shape === 'icon' && size === 'small',
                [styles.iconStandard]: shape === 'icon' && size === 'standard',
                [styles.iconLarge]: shape === 'icon' && size === 'large',
                [styles.standardSmall]: shape === 'standard' && size === 'small',
                [styles.standardStandard]: shape === 'standard' && size === 'standard',
                [styles.standardLarge]: shape === 'standard' && size === 'large',
            }), ref: ref }, restOfProps)));
    });
    Button.displayName = 'Button';

    const useStyles$1 = reactStyleSystem.createStyles(({ css }) => ({
        root: css `
    list-style-type: none;
    padding: 0;
    margin: 0;
  `,
    }));
    const List = React.forwardRef((props, ref) => {
        const _a = useStyles$1(props, 'ul' ), { Root, styles } = _a, restOfProps = __rest(_a, ["Root", "styles"]);
        return React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps));
    });
    List.displayName = 'List';

    const useStyles$2 = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    ${theme.fonts.body1};
    display: flex;
  `,
    }));
    const ListItem = React.forwardRef((props, ref) => {
        const _a = useStyles$2(props, 'li'), { Root, styles } = _a, restOfProps = __rest(_a, ["Root", "styles"]);
        return React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps));
    });
    ListItem.displayName = 'ListItem';

    function ThemeProvider({ theme, children }) {
        return (React__default.createElement(reactStyleSystem.ThemeProvider, { theme: theme },
            React__default.createElement(reactStyleSystem.ColorContextProvider, { color: theme.colors.accent, surface: theme.colors.surface }, children)));
    }

    const defaultTheme = {
        fonts: {
            button: reactStyleSystem.css `
      font-size: 0.75rem;
      font-weight: bold;
      text-transform: uppercase;
      margin: 0;
    `,
            caption: reactStyleSystem.css `
      font-size: 0.8rem;
      margin: 0;
    `,
            h1: reactStyleSystem.css `
      font-size: 160px;
      font-weight: bold;
      margin: 0;
    `,
            h2: reactStyleSystem.css `
      font-size: 100px;
      font-weight: bold;
      margin: 0;
    `,
            h3: reactStyleSystem.css `
      font-size: 60px;
      font-weight: bold;
      margin: 0;
    `,
            h4: reactStyleSystem.css `
      font-size: 24px;
      font-weight: bold;
      line-height: 1.5;
      margin: 0;
    `,
            h5: reactStyleSystem.css `
      font-size: 16px;
      font-weight: bold;
      line-height: 1.5;
      margin: 0;
    `,
            subtitle1: reactStyleSystem.css `
      font-size: 60px;
      font-weight: bold;
      color: #444;
      margin: 0;
    `,
            subtitle2: reactStyleSystem.css `
      font-size: 24px;
      font-weight: bold;
      color: #444;
      margin: 0;
    `,
            body1: reactStyleSystem.css `
      font-weight: 400;
      text-transform: none;
      font-size: 16px;
      margin: 0;
      line-height: 1.5;
    `,
            body2: reactStyleSystem.css `
      font-weight: 400;
      text-transform: none;
      font-size: 16px;
      margin: 0;
      color: #444;
    `,
        },
        colors: {
            brand: '#000',
            accent: '#2962ff',
            bland: '#ccc',
            danger: '#eb002b',
            warning: '#f56200',
            info: '#2962ff',
            surface: '#fff',
        },
        durations: {
            short: '100ms',
            standard: '250ms',
            long: '500ms',
        },
        shadows: {
            subtle: '0 0 10px 0 rgba(0,0,0,0.10)',
            standard: '0 0 20px 0 rgba(0,0,0,0.13)',
            emphasis: '0 0 30px 0 rgba(0,0,0,0.13)',
        },
        space: n => `${n * 16}px`,
        gap: n => `${n * 48}px`,
        block: n => `${n * 96}px`,
        zIndex: {
            appBar: 1100,
            drawer: 1200,
            modal: 1300,
            notification: 1400,
            tooltip: 1500,
        },
        breakpoints: {
            mobile: '375px',
            tablet: '768px',
            desktop: '1024px',
            desktopLarge: '1440p',
            up: (value) => `@media (min-width: ${polished.math(`${value} + 1px`)})`,
            down: (value) => `@media (max-width: ${value})`,
            between: (min, max) => `@media (max-width: ${min}) and (min-width: ${polished.math(`${max} - 1px`)})`,
        },
    };

    function isObject(maybe) {
        if (!maybe)
            return false;
        return typeof maybe === 'object';
    }
    /**
     * recursively merges a base object with an incoming object
     */
    function recursiveMerge(base, incoming = {}) {
        const keys = [...Object.keys(base), ...Object.keys(incoming)];
        const merged = keys.reduce((merged, key) => {
            const baseValue = base[key];
            const incomingValue = incoming[key];
            const mergedValue = isObject(baseValue)
                ? recursiveMerge(baseValue, incomingValue)
                : (incomingValue || baseValue);
            merged[key] = mergedValue;
            return merged;
        }, {});
        return merged;
    }
    function createTheme(partialTheme = {}) {
        return recursiveMerge(defaultTheme, partialTheme);
    }

    const useStyles$3 = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    ${theme.fonts.body1}
    align-items: flex-start;
    width: 100%;
    display: flex;
    flex-direction: column;
  `,
    }));
    const ListItemButton = React.forwardRef((props, ref) => {
        const theme = reactStyleSystem.useTheme();
        const _a = useStyles$3(props, Button), { Root, styles } = _a, restOfProps = __rest(_a, ["Root", "styles"]);
        return (React__default.createElement(Root, Object.assign({ ref: ref, variant: "ghost", color: theme.colors.bland }, restOfProps)));
    });
    ListItemButton.displayName = 'ListItemButton';

    const useStyles$4 = reactStyleSystem.createStyles(({ css, theme, color }) => ({
        root: css `
    color: ${color.readable};
    text-decoration: underline;
    transition: color ${theme.durations.standard};

    &:active {
      color: ${polished.transparentize(0.5, color.readable)};
    }
  `,
    }));
    const Anchor = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$4(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'a'), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
        return React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps));
    });
    Anchor.displayName = 'Anchor';

    var FormControlContext = React.createContext(null);

    const useStyles$5 = reactStyleSystem.createStyles(({ css }) => ({
        root: css `
    display: flex;
    flex-direction: column;
  `,
    }));
    const FormControl = React.forwardRef((props, ref) => {
        var _a, _b, _c, _d;
        const _e = useStyles$5(props, props.component || 'div'), { Root, styles, component, hasError = false, disabled = false } = _e, restOfProps = __rest(_e, ["Root", "styles", "component", "hasError", "disabled"]);
        const colorContext = reactStyleSystem.useColorContext();
        const theme = reactStyleSystem.useTheme();
        const color = (_b = (_a = props.color) !== null && _a !== void 0 ? _a : colorContext === null || colorContext === void 0 ? void 0 : colorContext.color) !== null && _b !== void 0 ? _b : theme.colors.accent;
        const surface = (_d = (_c = props.surface) !== null && _c !== void 0 ? _c : colorContext === null || colorContext === void 0 ? void 0 : colorContext.surface) !== null && _d !== void 0 ? _d : theme.colors.surface;
        const id = React.useMemo(() => `hui-${nanoId()}`, []);
        const [focused, setFocused] = React.useState(false);
        const formControlContextValue = React.useMemo(() => ({
            id,
            focused,
            setFocused,
            hasError,
            disabled,
        }), [disabled, focused, hasError, id]);
        return (React__default.createElement(reactStyleSystem.ColorContextProvider, { color: color, surface: surface },
            React__default.createElement(FormControlContext.Provider, { value: formControlContextValue },
                React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps)))));
    });
    FormControl.displayName = 'FormControl';

    const useStyles$6 = reactStyleSystem.createStyles(({ css, theme, color, surface }) => {
        const bland = reactStyleSystem.createReadablePalette(theme.colors.bland, surface);
        return {
            root: css `
      ${theme.fonts.body1};
      color: ${polished.readableColor(theme.colors.surface)};
      transition: color ${theme.durations.standard};
      cursor: pointer;
    `,
            focused: css `
      color: ${color.readable};
    `,
            hasError: css `
      color: ${theme.colors.danger};
    `,
            disabled: css `
      color: ${polished.transparentize(0.3, bland.readable)};
      cursor: not-allowed;
    `,
        };
    });
    const Label = React.forwardRef((props, ref) => {
        var _a, _b, _c;
        const _d = useStyles$6(props, props.component || 'label'), { Root, styles, component, id: incomingId, focused: incomingFocused, hasError: incomingHasError, disabled: incomingDisabled } = _d, restOfProps = __rest(_d, ["Root", "styles", "component", "id", "focused", "hasError", "disabled"]);
        const formControlContext = React.useContext(FormControlContext);
        const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
        const focused = (_a = incomingFocused !== null && incomingFocused !== void 0 ? incomingFocused : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.focused) !== null && _a !== void 0 ? _a : false;
        const hasError = (_b = incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) !== null && _b !== void 0 ? _b : false;
        const disabled = (_c = incomingDisabled !== null && incomingDisabled !== void 0 ? incomingDisabled : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) !== null && _c !== void 0 ? _c : false;
        return (React__default.createElement(Root, Object.assign({ className: classNames({
                [styles.focused]: focused,
                [styles.hasError]: hasError,
                [styles.disabled]: disabled,
            }), ref: ref, htmlFor: id }, restOfProps)));
    });
    Label.displayName = 'Label';

    const useStyles$7 = reactStyleSystem.createStyles(({ css, theme, color, surface }) => {
        const bland = reactStyleSystem.createReadablePalette(theme.colors.bland, surface);
        const danger = reactStyleSystem.createReadablePalette(theme.colors.danger, surface);
        return {
            root: css `
      ${theme.fonts.body1};
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border: none;
      outline: none;
      appearance: none;
      background: none;
      margin: ${theme.space(0.5)} 0;

      &:disabled {
        cursor: not-allowed;
      }
    `,
            filled: css `
      background-color: ${polished.transparentize(0.8, bland.decorative)};
      color: ${polished.readableColor(surface)};
      transition: background-color ${theme.durations.standard};

      &:focus {
        background-color: ${polished.transparentize(0.92, color.decorative)};
        color: ${polished.readableColor(surface)};
      }
      &:hover {
        background-color: ${polished.transparentize(0.9, color.decorative)};
        color: ${polished.readableColor(surface)};
      }
      &:disabled {
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }
    `,
            filledHasError: css `
      background-color: ${polished.transparentize(0.9, danger.decorative)};
      color: ${polished.readableColor(surface)};
      &:focus {
        background-color: ${polished.transparentize(0.85, danger.decorative)};
      }
      &:not([disabled]):hover {
        background-color: ${polished.transparentize(0.87, danger.decorative)};
      }
    `,
            outlined: css `
      background-color: ${surface};
      transition: border ${theme.durations.standard},
        background-color ${theme.durations.standard};
      border: 2px solid ${bland.decorative};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:hover {
        border: 2px solid ${polished.transparentize(0.3, color.decorative)};
      }
      &:disabled {
        border: 2px solid ${polished.transparentize(0.7, bland.decorative)};
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }
    `,
            outlinedHasError: css `
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${polished.transparentize(0.3, danger.decorative)};
      }
    `,
        };
    });
    const TextInput = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$7(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'input'), { Root, styles, variant = 'outlined', hasError: incomingHasError, disabled: incomingDisabled, onFocus, onBlur, type = 'text' } = _b, restOfProps = __rest(_b, ["Root", "styles", "variant", "hasError", "disabled", "onFocus", "onBlur", "type"]);
        const formControlContext = React.useContext(FormControlContext);
        const id = formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
        const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);
        const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
        const handleFocus = (e) => {
            if (onFocus) {
                onFocus(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(true);
            }
        };
        const handleBlur = (e) => {
            if (onBlur) {
                onBlur(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(false);
            }
        };
        return (React__default.createElement(Root, Object.assign({ ref: ref, id: id, type: type, disabled: disabled, className: classNames({
                [styles.outlined]: variant === 'outlined',
                [styles.outlinedHasError]: variant === 'outlined' && hasError,
                [styles.filled]: variant === 'filled',
                [styles.filledHasError]: variant === 'filled' && hasError,
            }), onFocus: handleFocus, onBlur: handleBlur }, restOfProps)));
    });
    TextInput.displayName = 'TextInput';

    const useStyles$8 = reactStyleSystem.createStyles(({ css, color, theme, surface }) => {
        const bland = reactStyleSystem.createReadablePalette(theme.colors.bland, surface);
        return {
            root: css `
      ${theme.fonts.caption};
      transition: color ${theme.durations.standard};
    `,
            focused: css `
      color: ${color.readable};
    `,
            hasError: css `
      color: ${theme.colors.danger};
    `,
            disabled: css `
      color: ${polished.transparentize(0.3, bland.readable)};
      cursor: not-allowed;
    `,
        };
    });
    const HelperText = React.forwardRef((props, ref) => {
        var _a, _b, _c, _d;
        const formControlContext = React.useContext(FormControlContext);
        const _e = useStyles$8(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'div'), { Root, styles, component, focused: incomingFocused, hasError: incomingHasError, disabled: incomingDisabled } = _e, restOfProps = __rest(_e, ["Root", "styles", "component", "focused", "hasError", "disabled"]);
        const focused = (_b = incomingFocused !== null && incomingFocused !== void 0 ? incomingFocused : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.focused) !== null && _b !== void 0 ? _b : false;
        const hasError = (_c = incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) !== null && _c !== void 0 ? _c : false;
        const disabled = (_d = incomingDisabled !== null && incomingDisabled !== void 0 ? incomingDisabled : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) !== null && _d !== void 0 ? _d : false;
        return (React__default.createElement(Root, Object.assign({ className: classNames({
                [styles.focused]: focused,
                [styles.hasError]: hasError,
                [styles.disabled]: disabled,
            }), ref: ref }, restOfProps)));
    });
    HelperText.displayName = 'HelperText';

    const useStyles$9 = reactStyleSystem.createStyles(({ css, theme, color, surface }) => {
        const bland = reactStyleSystem.createReadablePalette(theme.colors.bland, surface);
        const danger = reactStyleSystem.createReadablePalette(theme.colors.danger, surface);
        return {
            root: css `
      ${theme.fonts.body1};
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border: none;
      outline: none;
      appearance: none;
      background: none;
      margin: ${theme.space(0.5)} 0;

      &:disabled {
        cursor: not-allowed;
      }
    `,
            filled: css `
      background-color: ${polished.transparentize(0.8, bland.decorative)};
      color: ${polished.readableColor(surface)};
      transition: background-color ${theme.durations.standard};

      &:focus {
        background-color: ${polished.transparentize(0.92, color.decorative)};
        color: ${polished.readableColor(surface)};
      }
      &:hover {
        background-color: ${polished.transparentize(0.9, color.decorative)};
        color: ${polished.readableColor(surface)};
      }
      &:disabled {
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }
    `,
            filledHasError: css `
      background-color: ${polished.transparentize(0.9, danger.decorative)};
      color: ${polished.readableColor(surface)};
      &:focus {
        background-color: ${polished.transparentize(0.85, danger.decorative)};
      }
      &:not([disabled]):hover {
        background-color: ${polished.transparentize(0.87, danger.decorative)};
      }
    `,
            outlined: css `
      background-color: ${surface};
      transition: border ${theme.durations.standard},
        background-color ${theme.durations.standard};
      border: 2px solid ${bland.decorative};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:hover {
        border: 2px solid ${polished.transparentize(0.3, color.decorative)};
      }
      &:disabled {
        border: 2px solid ${polished.transparentize(0.7, bland.decorative)};
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }
    `,
            outlinedHasError: css `
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${polished.transparentize(0.3, danger.decorative)};
      }
    `,
        };
    });
    const TextArea = React.forwardRef((props, ref) => {
        var _a, _b, _c;
        const _d = useStyles$9(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'textarea'), { Root, styles, variant = 'outlined', focused: _focused, hasError: incomingHasError, disabled: incomingDisabled, onFocus, onBlur } = _d, restOfProps = __rest(_d, ["Root", "styles", "variant", "focused", "hasError", "disabled", "onFocus", "onBlur"]);
        const formControlContext = React.useContext(FormControlContext);
        const id = formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
        const hasError = (_b = incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) !== null && _b !== void 0 ? _b : false;
        const disabledFromFormControl = (_c = formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) !== null && _c !== void 0 ? _c : false;
        const disabled = incomingDisabled || disabledFromFormControl;
        const handleFocus = (e) => {
            if (onFocus) {
                onFocus(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(true);
            }
        };
        const handleBlur = (e) => {
            if (onBlur) {
                onBlur(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(false);
            }
        };
        return (React__default.createElement(Root, Object.assign({ ref: ref, id: id, disabled: disabled, className: classNames({
                [styles.outlined]: variant === 'outlined',
                [styles.outlinedHasError]: variant === 'outlined' && hasError,
                [styles.filled]: variant === 'filled',
                [styles.filledHasError]: variant === 'filled' && hasError,
            }), onFocus: handleFocus, onBlur: handleBlur }, restOfProps)));
    });
    TextArea.displayName = 'TextArea';

    /**
     * this is a fontawesome 5 free icon
     * @license https://fontawesome.com/license
     */
    // TODO: add types
    const CheckIcon = React.forwardRef((props, ref) => {
        return (React__default.createElement("svg", Object.assign({ ref: ref, "aria-hidden": "true", viewBox: "0 0 512 512" }, props),
            React__default.createElement("path", { fill: "currentColor", d: "M173.898 439.404l-166.4-166.4c-9.997-9.997-9.997-26.206 0-36.204l36.203-36.204c9.997-9.998 26.207-9.998 36.204 0L192 312.69 432.095 72.596c9.997-9.997 26.207-9.997 36.204 0l36.203 36.204c9.997 9.997 9.997 26.206 0 36.204l-294.4 294.401c-9.998 9.997-26.207 9.997-36.204-.001z" })));
    });
    CheckIcon.displayName = 'CheckIcon';

    const useStyles$a = reactStyleSystem.createStyles(({ css, color, theme, surface }) => {
        const bland = reactStyleSystem.createReadablePalette(theme.colors.bland, surface);
        const danger = reactStyleSystem.createReadablePalette(theme.colors.danger, surface);
        const width = theme.space(2);
        const height = theme.space(2);
        const widthSmall = theme.space(1.5);
        const heightSmall = theme.space(1.5);
        const widthLarge = theme.space(2.5);
        const heightLarge = theme.space(2.5);
        return {
            root: css `
      position: relative;
      margin: ${theme.space(0.5)} 0;
      background-color: ${surface};
    `,
            hasError: css `
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .checkbox:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      & .checkbox:not([disabled]):hover ~ .facade {
        border: 2px solid ${polished.transparentize(0.3, danger.decorative)};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      & .checkbox:active ~ .facade {
        background-color: ${polished.transparentize(0.9, danger.decorative)};
      }
    `,
            checkbox: css `
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:hover ~ .facade {
        border: 2px solid ${polished.transparentize(0.3, color.decorative)};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:active ~ .facade {
        background-color: ${polished.transparentize(0.9, color.decorative)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${polished.transparentize(0.7, bland.decorative)};
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .icon {
        opacity: 1;
      }
    `,
            checkboxStandard: css `
      width: ${width};
      height: ${height};
    `,
            checkboxSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
            checkboxLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
            facade: css `
      width: ${width};
      height: ${height};
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.durations.standard},
        background-color ${theme.durations.standard};
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
    `,
            facadeSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
            facadeStandard: css `
      width: ${width};
      height: ${height};
    `,
            facadeLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
            icon: css `
      width: 1rem;
      height: 1rem;
      fill: currentColor;
      opacity: 0;
      pointer-events: none;
    `,
        };
    });
    const Checkbox = React.forwardRef((props, ref) => {
        const _a = useStyles$a(props), { Root, styles, id: incomingId, hasError: incomingHasError, disabled: incomingDisabled, onFocus, onBlur, inputRef, icon: Icon = CheckIcon, size = 'standard' } = _a, restOfProps = __rest(_a, ["Root", "styles", "id", "hasError", "disabled", "onFocus", "onBlur", "inputRef", "icon", "size"]);
        const formControlContext = React.useContext(FormControlContext);
        const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
        const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
        const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);
        const handleFocus = (e) => {
            if (onFocus) {
                onFocus(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(true);
            }
        };
        const handleBlur = (e) => {
            if (onBlur) {
                onBlur(e);
            }
            if (formControlContext) {
                formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.setFocused(false);
            }
        };
        return (React__default.createElement(Root, { ref: ref, className: classNames({
                [styles.hasError]: hasError,
            }) },
            React__default.createElement("input", Object.assign({ type: "checkbox", id: id, ref: inputRef, className: classNames('checkbox', styles.checkbox, {
                    [styles.checkboxSmall]: size === 'small',
                    [styles.checkboxStandard]: size === 'standard',
                    [styles.checkboxLarge]: size === 'large',
                }), onFocus: handleFocus, onBlur: handleBlur, disabled: disabled }, restOfProps)),
            React__default.createElement("div", { "aria-hidden": "true", className: classNames('facade', styles.facade, {
                    [styles.facadeSmall]: size === 'small',
                    [styles.facadeStandard]: size === 'standard',
                    [styles.facadeLarge]: size === 'large',
                }) },
                React__default.createElement(Icon, { className: classNames('icon', styles.icon) }))));
    });
    Checkbox.displayName = 'Checkbox';

    var RadioGroupContext = React.createContext(null);

    // TODO: add types
    const CircleIcon = React.forwardRef((props, ref) => {
        return (React__default.createElement("svg", Object.assign({ ref: ref, "aria-hidden": "true", viewBox: "0 0 512 512" }, props),
            React__default.createElement("circle", { cx: "256", cy: "256", fill: "currentColor", r: "192" })));
    });
    CircleIcon.displayName = 'CircleIcon';

    const useStyles$b = reactStyleSystem.createStyles(({ css, color, theme, surface }) => {
        const bland = reactStyleSystem.createReadablePalette(theme.colors.bland, surface);
        const danger = reactStyleSystem.createReadablePalette(theme.colors.danger, surface);
        const width = theme.space(2);
        const height = theme.space(2);
        const widthSmall = theme.space(1.5);
        const heightSmall = theme.space(1.5);
        const widthLarge = theme.space(2.5);
        const heightLarge = theme.space(2.5);
        return {
            root: css `
      position: relative;
      margin: ${theme.space(0.5)} 0;
    `,
            hasError: css `
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .radio:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      & .radio:not([disabled]):hover ~ .facade {
        border: 2px solid ${polished.transparentize(0.3, danger.decorative)};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      & .radio:active ~ .facade {
        background-color: ${polished.transparentize(0.9, danger.decorative)};
      }
    `,
            radio: css `
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:hover ~ .facade {
        border: 2px solid ${polished.transparentize(0.3, color.decorative)};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:active ~ .facade {
        background-color: ${polished.transparentize(0.9, color.decorative)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${polished.transparentize(0.7, bland.decorative)};
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .icon {
        opacity: 1;
      }
    `,
            radioStandard: css `
      width: ${width};
      height: ${height};
    `,
            radioSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
            radioLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
            facade: css `
      width: ${width};
      height: ${height};
      border-radius: 99999px;
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.durations.standard},
        background-color ${theme.durations.standard};
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
    `,
            facadeSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
            facadeStandard: css `
      width: ${width};
      height: ${height};
    `,
            facadeLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
            icon: css `
      width: 1rem;
      height: 1rem;
      fill: currentColor;
      opacity: 0;
      pointer-events: none;
    `,
        };
    });
    const Radio = React.forwardRef((props, ref) => {
        const _a = useStyles$b(props), { Root, styles, id: incomingId, hasError: incomingHasError, disabled: incomingDisabled, checked: incomingChecked, name: incomingName, value, onFocus, onBlur, onChange, inputRef, icon: Icon = CircleIcon, size = 'standard' } = _a, restOfProps = __rest(_a, ["Root", "styles", "id", "hasError", "disabled", "checked", "name", "value", "onFocus", "onBlur", "onChange", "inputRef", "icon", "size"]);
        const formControlContext = React.useContext(FormControlContext);
        const radioGroupContext = React.useContext(RadioGroupContext);
        const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
        const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
        const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);
        const checkedFromContext = radioGroupContext
            ? radioGroupContext.value === value
            : undefined;
        const checked = incomingChecked !== null && incomingChecked !== void 0 ? incomingChecked : checkedFromContext;
        const name = incomingName !== null && incomingName !== void 0 ? incomingName : radioGroupContext === null || radioGroupContext === void 0 ? void 0 : radioGroupContext.name;
        const handleFocus = (e) => {
            if (onFocus) {
                onFocus(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(true);
            }
        };
        const handleBlur = (e) => {
            if (onBlur) {
                onBlur(e);
            }
            if (formControlContext) {
                formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.setFocused(false);
            }
        };
        const handleChange = (e) => {
            if (onChange) {
                onChange(e);
            }
            if (radioGroupContext) {
                radioGroupContext.handleChange(e);
            }
        };
        return (React__default.createElement(Root, { ref: ref, className: classNames({
                [styles.hasError]: hasError,
            }) },
            React__default.createElement("input", Object.assign({ type: "radio", id: id, ref: inputRef, value: value, name: name, checked: checked, className: classNames('radio', styles.radio, {
                    [styles.radioSmall]: size === 'small',
                    [styles.radioStandard]: size === 'standard',
                    [styles.radioLarge]: size === 'large',
                }), onFocus: handleFocus, onBlur: handleBlur, onChange: handleChange, disabled: disabled }, restOfProps)),
            React__default.createElement("div", { "aria-hidden": "true", className: classNames('facade', styles.facade, {
                    [styles.facadeSmall]: size === 'small',
                    [styles.facadeStandard]: size === 'standard',
                    [styles.facadeLarge]: size === 'large',
                }) },
                React__default.createElement(Icon, { className: classNames('icon', styles.icon) }))));
    });
    Radio.displayName = 'Radio';

    const useStyles$c = reactStyleSystem.createStyles(({ css }) => ({
        root: css ``,
    }));
    const RadioGroup = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$c(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'div'), { Root, styles, component, value: incomingValue, onChange } = _b, restOfProps = __rest(_b, ["Root", "styles", "component", "value", "onChange"]);
        const name = React.useMemo(() => `hui-group-${nanoId()}`, []);
        const [stateValue, setStateValue] = React.useState('');
        const value = incomingValue !== null && incomingValue !== void 0 ? incomingValue : stateValue;
        // pull the reference of onChange on invocation
        const onChangeRef = React.useRef(onChange);
        const handleChange = React.useCallback((e) => {
            const onChange = onChangeRef.current;
            if (onChange) {
                onChange(e);
            }
            setStateValue(e.currentTarget.value);
        }, []);
        const contextValue = React.useMemo(() => ({
            name,
            value,
            handleChange,
        }), [handleChange, name, value]);
        return (React__default.createElement(RadioGroupContext.Provider, { value: contextValue },
            React__default.createElement(Root, Object.assign({ ref: ref, role: "radiogroup" }, restOfProps))));
    });
    RadioGroup.displayName = 'RadioGroup';

    const useStyles$d = reactStyleSystem.createStyles(({ css, color, theme, surface }) => {
        const bland = reactStyleSystem.createReadablePalette(theme.colors.bland, surface);
        const danger = reactStyleSystem.createReadablePalette(theme.colors.danger, surface);
        const width = theme.space(3.5);
        const height = theme.space(2);
        const widthSmall = theme.space(2.75);
        const heightSmall = theme.space(1.5);
        const widthLarge = theme.space(4);
        const heightLarge = theme.space(2.5);
        return {
            root: css `
      position: relative;
      margin: ${theme.space(0.5)} 0;
      background-color: ${surface};
      border-radius: 99999px;
    `,
            hasError: css `
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .switch:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      & .switch:not([disabled]):hover ~ .facade {
        border: 2px solid ${polished.transparentize(0.3, danger.decorative)};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      & .switch:active ~ .facade {
        background-color: ${polished.transparentize(0.9, danger.decorative)};
      }

      & .switch:checked ~ .facade {
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }

      & .switch:checked ~ .facade .dot {
        fill: ${danger.decorative};
      }
    `,
            switch: css `
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:hover ~ .facade {
        border: 2px solid ${polished.transparentize(0.3, color.decorative)};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:active ~ .facade {
        background-color: ${polished.transparentize(0.9, color.decorative)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${polished.transparentize(0.7, bland.decorative)};
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .dot {
        cx: 125;
        fill: ${color.decorative};
      }

      &:disabled ~ .facade .dot {
        fill: ${bland.decorative};
      }

      &:checked ~ .facade {
        /* border: 2px solid ${bland.decorative}; */
        background-color: ${polished.transparentize(0.9, color.decorative)};
      }
    `,
            switchStandard: css `
      width: ${width};
      height: ${height};
    `,
            switchSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
            switchLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
            facade: css `
      width: ${width};
      height: ${height};
      border-radius: 99999px;
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.durations.standard},
        background-color ${theme.durations.standard};
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
    `,
            facadeSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
            facadeStandard: css `
      width: ${width};
      height: ${height};
    `,
            facadeLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
            icon: css `
      width: ${width};
      height: ${height};
      fill: currentColor;
      pointer-events: none;
    `,
            dot: css `
      transition: cx ${theme.durations.standard},
        fill ${theme.durations.standard};
    `,
            dotSmall: css `
      r: 30;
    `,
            dotStandard: css `
      r: 35;
    `,
            dotLarge: css `
      r: 40;
    `,
        };
    });
    const Switch = React.forwardRef((props, ref) => {
        const _a = useStyles$d(props), { Root, styles, id: incomingId, hasError: incomingHasError, disabled: incomingDisabled, onFocus, onBlur, inputRef, size = 'standard' } = _a, restOfProps = __rest(_a, ["Root", "styles", "id", "hasError", "disabled", "onFocus", "onBlur", "inputRef", "size"]);
        const formControlContext = React.useContext(FormControlContext);
        const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
        const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
        const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);
        const handleFocus = (e) => {
            if (onFocus) {
                onFocus(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(true);
            }
        };
        const handleBlur = (e) => {
            if (onBlur) {
                onBlur(e);
            }
            if (formControlContext) {
                formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.setFocused(false);
            }
        };
        return (React__default.createElement(Root, { ref: ref, className: classNames({
                [styles.hasError]: hasError,
            }) },
            React__default.createElement("input", Object.assign({ type: "checkbox", id: id, ref: inputRef, className: classNames('switch', styles.switch, {
                    [styles.switchSmall]: size === 'small',
                    [styles.switchStandard]: size === 'standard',
                    [styles.switchLarge]: size === 'large',
                }), onFocus: handleFocus, onBlur: handleBlur, disabled: disabled }, restOfProps)),
            React__default.createElement("div", { "aria-hidden": "true", className: classNames('facade', styles.facade, {
                    [styles.facadeSmall]: size === 'small',
                    [styles.facadeStandard]: size === 'standard',
                    [styles.facadeLarge]: size === 'large',
                }) },
                React__default.createElement("svg", { className: styles.icon, viewBox: "0 0 175 100" },
                    React__default.createElement("circle", { className: classNames('dot', styles.dot, {
                            [styles.dotSmall]: size === 'small',
                            [styles.dotStandard]: size === 'standard',
                            [styles.dotLarge]: size === 'large',
                        }), cx: "50", cy: "50" })))));
    });
    Switch.displayName = 'Switch';

    /**
     * this is a fontawesome 5 free icon
     * @license https://fontawesome.com/license
     */
    // TODO: add types
    const AngleDownIcon = React.forwardRef((props, ref) => {
        return (React__default.createElement("svg", Object.assign({ ref: ref, "aria-hidden": "true", viewBox: "0 0 320 512" }, props),
            React__default.createElement("path", { fill: "currentColor", d: "M143 352.3L7 216.3c-9.4-9.4-9.4-24.6 0-33.9l22.6-22.6c9.4-9.4 24.6-9.4 33.9 0l96.4 96.4 96.4-96.4c9.4-9.4 24.6-9.4 33.9 0l22.6 22.6c9.4 9.4 9.4 24.6 0 33.9l-136 136c-9.2 9.4-24.4 9.4-33.8 0z" })));
    });
    AngleDownIcon.displayName = 'AngleDownIcon';

    const useStyles$e = reactStyleSystem.createStyles(({ css, theme, color, surface }) => {
        const bland = reactStyleSystem.createReadablePalette(theme.colors.bland, surface);
        const danger = reactStyleSystem.createReadablePalette(theme.colors.danger, surface);
        return {
            root: css `
      position: relative;
      margin: ${theme.space(0.5)} 0;
    `,
            select: css `
      ${theme.fonts.body1};
      width: 100%;
      height: 100%;
      appearance: none;
      outline: none;
      border: none;
      background: none;
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border-radius: 0;
      cursor: pointer;

      &:disabled {
        cursor: not-allowed;
      }
    `,
            selectFilled: css `
      background-color: ${polished.transparentize(0.8, bland.decorative)};
      color: ${polished.readableColor(surface)};
      transition: background-color ${theme.durations.standard};

      &:focus {
        background-color: ${polished.transparentize(0.92, color.decorative)};
        color: ${polished.readableColor(surface)};
      }
      &:hover {
        background-color: ${polished.transparentize(0.9, color.decorative)};
        color: ${polished.readableColor(surface)};
      }
      &:disabled {
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }
    `,
            selectFilledHasError: css `
      background-color: ${polished.transparentize(0.9, danger.decorative)};
      color: ${polished.readableColor(surface)};
      &:focus {
        background-color: ${polished.transparentize(0.85, danger.decorative)};
      }
      &:not([disabled]):hover {
        background-color: ${polished.transparentize(0.87, danger.decorative)};
      }
    `,
            selectOutlined: css `
      background-color: ${surface};
      transition: border ${theme.durations.standard},
        background-color ${theme.durations.standard},
        color ${theme.durations.standard};
      border: 2px solid ${bland.decorative};
      color: ${polished.readableColor(surface)};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${polished.transparentize(0.93, color.decorative)};
      }
      &:hover {
        border: 2px solid ${polished.transparentize(0.3, color.decorative)};
      }
      &:disabled {
        border: 2px solid ${polished.transparentize(0.7, bland.decorative)};
        background-color: ${polished.transparentize(0.9, bland.decorative)};
      }
    `,
            selectedOutlinedHasError: css `
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${polished.transparentize(0.93, danger.decorative)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${polished.transparentize(0.3, danger.decorative)};
      }
    `,
            icon: css `
      pointer-events: none;
      width: ${theme.space(1)};
      height: ${theme.space(1)};
      position: absolute;
      right: ${theme.space(0.75)};
      top: 50%;
      transform: translateY(-50%);
    `,
        };
    });
    const Select = React.forwardRef((props, ref) => {
        var _a;
        const formControlContext = React.useContext(FormControlContext);
        const _b = useStyles$e(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'div'), { Root, id: incomingId, styles, component, selectRef, disabled: incomingDisabled, hasError: incomingHasError, onFocus, onBlur, variant = 'outlined' } = _b, restOfProps = __rest(_b, ["Root", "id", "styles", "component", "selectRef", "disabled", "hasError", "onFocus", "onBlur", "variant"]);
        const disabled = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.disabled) || Boolean(incomingDisabled);
        const hasError = Boolean(formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.hasError) || Boolean(incomingHasError);
        const id = incomingId !== null && incomingId !== void 0 ? incomingId : formControlContext === null || formControlContext === void 0 ? void 0 : formControlContext.id;
        const handleFocus = (e) => {
            if (onFocus) {
                onFocus(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(true);
            }
        };
        const handleBlur = (e) => {
            if (onBlur) {
                onBlur(e);
            }
            if (formControlContext) {
                formControlContext.setFocused(false);
            }
        };
        return (React__default.createElement(Root, { ref: ref },
            React__default.createElement("select", Object.assign({ id: id, ref: selectRef, className: classNames(styles.select, {
                    [styles.selectFilled]: variant === 'filled',
                    [styles.selectFilledHasError]: variant === 'filled' && hasError,
                    [styles.selectOutlined]: variant === 'outlined',
                    [styles.selectedOutlinedHasError]: variant === 'outlined' && hasError,
                }), disabled: disabled, onFocus: handleFocus, onBlur: handleBlur }, restOfProps)),
            React__default.createElement(AngleDownIcon, { className: styles.icon })));
    });
    Select.displayName = 'Select';

    const useStyles$f = reactStyleSystem.createStyles(({ css, theme, color }) => ({
        // button base styles
        root: css `
    ${theme.fonts.caption}
    font-weight: bold;

    display: inline-flex;
    justify-content: center;
    align-items: center;

    appearance: none;
    outline: none;
    background: transparent;
    border: none;
    text-decoration: none;
    border-radius: 999999px;
    min-height: 34px;

    &:disabled {
      cursor: not-allowed;
    }

    padding: ${theme.space(0.5)} ${theme.space(0.75)};

    & > *:not(:last-child) {
      margin-right: ${theme.space(0.5)};
    }

    & > .hui-button {
      margin-top: ${`-${theme.space(0.5)}`};
      margin-bottom: ${`-${theme.space(0.5)}`};
      margin-right: ${`-${theme.space(0.75)}`};
      margin-left: ${`-${theme.space(0.25)}`};
    }
  `,
        // variants
        outlined: css `
    border: 1px solid ${color.readable};
    color: ${color.readable};

    transition: background-color ${theme.durations.standard},
      border ${theme.durations.standard};

    &:disabled {
      &,
      &:hover,
      &:focus {
        color: ${polished.transparentize(0.4, color.readable)};
        background-color: transparent;
      }
    }
  `,
        outlinedClickable: css `
    cursor: pointer;
    &:focus {
      background-color: ${polished.transparentize(0.92, color.decorative)};
    }
    &:hover {
      background-color: ${polished.transparentize(0.9, color.decorative)};
    }
    &:active {
      background-color: ${polished.transparentize(0.8, color.decorative)};
    }
  `,
        filled: css `
    background-color: ${color.decorative};
    color: ${polished.readableColor(color.decorative)};
    border: 2px solid transparent;
    transition: background-color ${theme.durations.standard},
      border ${theme.durations.standard};

    &:disabled {
      background-color: ${polished.transparentize(0.5, color.decorative)};
      border: 2px solid transparent;
      color: ${polished.transparentize(0.2, polished.readableColor(color.decorative))};
    }
  `,
        filledClickable: css `
    cursor: pointer;
    &:focus {
      background-color: ${polished.transparentize(0.18, color.decorative)};
      border: 2px solid ${polished.transparentize(0.7, '#fff')};
    }
    &:hover {
      background-color: ${polished.transparentize(0.23, color.decorative)};
    }
    &:active {
      background-color: ${polished.transparentize(0.3, color.decorative)};
      border: 2px solid ${polished.transparentize(0.5, '#fff')};
    }
  `,
    }));
    const Chip = React.forwardRef((props, ref) => {
        var _a, _b, _c;
        const component = (((_a = props.component) !== null && _a !== void 0 ? _a : props.clickable) ? 'button'
            : 'div');
        const _d = useStyles$f(props, component), { Root, styles, component: _component, clickable, variant = 'outlined' } = _d, restOfProps = __rest(_d, ["Root", "styles", "component", "clickable", "variant"]);
        const colorContext = reactStyleSystem.useColorContext();
        let color = (_b = props.color) !== null && _b !== void 0 ? _b : colorContext === null || colorContext === void 0 ? void 0 : colorContext.color;
        let surface = (_c = props.surface) !== null && _c !== void 0 ? _c : colorContext === null || colorContext === void 0 ? void 0 : colorContext.surface;
        if (variant === 'filled') {
            surface = color;
            color = polished.readableColor(surface);
        }
        return (React__default.createElement(reactStyleSystem.ColorContextProvider, { surface: surface, color: color },
            React__default.createElement(Root, Object.assign({ className: classNames({
                    [styles.outlined]: variant === 'outlined',
                    [styles.outlinedClickable]: variant === 'outlined' && clickable,
                    [styles.filled]: variant === 'filled',
                    [styles.filledClickable]: variant === 'filled' && clickable,
                }), ref: ref }, restOfProps))));
    });
    Chip.displayName = 'Chip';

    const useStyles$g = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    width: ${theme.space(1.5)};
    height: ${theme.space(1.5)};
    margin-top: ${`-${theme.space(0.5)}`};
    margin-bottom: ${`-${theme.space(0.5)}`};
    margin-left: ${`-${theme.space(0.5)}`};
    border-radius: 99999px;
    overflow: hidden;

    display: flex;
    align-items: center;
    justify-content: center;

    & > img {
      width: 100%;
      height: 100%;
      object-fit: cover;
    }

    & > svg {
      width: ${theme.space(1)};
      height: ${theme.space(1)};
    }
  `,
    }));
    const ChipThumbnail = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$g(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'div'), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
        return React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps));
    });
    ChipThumbnail.displayName = 'ChipThumbnail';

    /**
     * this is a fontawesome 5 free icon
     * @license https://fontawesome.com/license
     */
    // TODO: add types
    const TimesIcon = React.forwardRef((props, ref) => {
        return (React__default.createElement("svg", Object.assign({ ref: ref, "aria-hidden": "true", viewBox: "0 0 352 512" }, props),
            React__default.createElement("path", { fill: "currentColor", d: "M242.72 256l100.07-100.07c12.28-12.28 12.28-32.19 0-44.48l-22.24-22.24c-12.28-12.28-32.19-12.28-44.48 0L176 189.28 75.93 89.21c-12.28-12.28-32.19-12.28-44.48 0L9.21 111.45c-12.28 12.28-12.28 32.19 0 44.48L109.28 256 9.21 356.07c-12.28 12.28-12.28 32.19 0 44.48l22.24 22.24c12.28 12.28 32.2 12.28 44.48 0L176 322.72l100.07 100.07c12.28 12.28 32.2 12.28 44.48 0l22.24-22.24c12.28-12.28 12.28-32.19 0-44.48L242.72 256z" })));
    });
    TimesIcon.displayName = 'TimesIcon';

    function delay(milliseconds) {
        return new Promise(resolve => setTimeout(() => resolve('delay'), milliseconds));
    }

    // https://usehooks.com/useDebounce/
    function useDebounce(value, delay) {
        // State and setters for debounced value
        const [debouncedValue, setDebouncedValue] = React.useState(value);
        React.useEffect(() => {
            // Update debounced value after delay
            const handler = setTimeout(() => {
                setDebouncedValue(value);
            }, delay);
            // Cancel the timeout if value changes (also on delay change or unmount)
            // This is how we prevent debounced value from updating if value is changed ...
            // .. within the delay period. Timeout gets cleared and restarted.
            return () => {
                clearTimeout(handler);
            };
        }, [value, delay]);
        return debouncedValue;
    }

    const useStyles$h = reactStyleSystem.createStyles(({ css, theme, surface }) => ({
        root: css `
    ${theme.fonts.caption};
    position: absolute;
    pointer-events: auto;
    padding: ${theme.space(0.5)} ${theme.space(1)};
    min-width: ${theme.block(1)};
    text-align: center;
    color: ${polished.readableColor(polished.readableColor(surface))};
    background-color: ${polished.transparentize(0.2, polished.readableColor(surface))};
    z-index: ${theme.zIndex.tooltip};
  `,
        top: css `
    transform: translate(-50%, -100%);
  `,
        left: css `
    transform: translate(-100%, -50%);
  `,
        bottom: css `
    transform: translate(-50%, 0);
  `,
        right: css `
    transform: translate(0, -50%);
  `,
        tooltipContainer: css `
    position: fixed;
    pointer-events: none;
    width: 100vw;
    height: 100vh;
    top: 0;
    left: 0;
  `,
    }));
    function Tooltip(props) {
        const { Root, styles, title, children, position = 'top' } = useStyles$h(props);
        const rootRef = React.useRef(null);
        const [isMouseOverRoot, setIsMouseOverRoot] = React.useState(false);
        const [isMouseOverTooltip, setIsMouseOverTooltip] = React.useState(false);
        // TODO: this would be better as one object
        const [top, setTop] = React.useState(0);
        const [left, setLeft] = React.useState(0);
        const [width, setWidth] = React.useState(0);
        const [height, setHeight] = React.useState(0);
        const margin = 8;
        let tooltipTop;
        let tooltipLeft;
        const topCenter = top + height / 2;
        const leftCenter = left + width / 2;
        if (position === 'left') {
            tooltipTop = topCenter;
            tooltipLeft = left - margin;
        }
        else if (position === 'right') {
            tooltipTop = topCenter;
            tooltipLeft = left + width + margin;
        }
        else if (position === 'bottom') {
            tooltipTop = top + height + margin;
            tooltipLeft = leftCenter;
        }
        else {
            // position === 'top'
            tooltipTop = top - margin;
            tooltipLeft = leftCenter;
        }
        const [tooltipContainer, setTooltipContainer,] = React.useState(null);
        const isMouseOver = isMouseOverRoot || isMouseOverTooltip;
        const debouncedMouseOver = useDebounce(isMouseOver, 500);
        // remove the tooltip container on scroll
        React.useEffect(() => {
            const handleScroll = () => {
                setTooltipContainer(null);
            };
            document.addEventListener('scroll', handleScroll);
            return () => {
                document.removeEventListener('scroll', handleScroll);
            };
        }, []);
        // updates the coordinates of the tooltip
        React.useEffect(() => {
            let element = rootRef.current;
            if (!element)
                return;
            (() => __awaiter(this, void 0, void 0, function* () {
                while (debouncedMouseOver && !!tooltipContainer && !!element) {
                    element = rootRef.current;
                    if (!element)
                        continue;
                    const { top, left, width, height } = element.getBoundingClientRect();
                    setTop(top);
                    setLeft(left);
                    setWidth(width);
                    setHeight(height);
                    yield delay(100);
                }
            }))();
        }, [debouncedMouseOver, rootRef, tooltipContainer]);
        // adds container to the DOM when the user mouses over
        React.useEffect(() => {
            if (!debouncedMouseOver)
                return;
            const tooltipContainer = document.createElement('div');
            tooltipContainer.classList.add(styles.tooltipContainer);
            document.body.appendChild(tooltipContainer);
            setTooltipContainer(tooltipContainer);
            return () => {
                document.body.removeChild(tooltipContainer);
                setTooltipContainer(null);
            };
        }, [debouncedMouseOver, styles]);
        const handleEnterRoot = () => {
            setIsMouseOverRoot(true);
        };
        const handleLeaveRoot = () => {
            setIsMouseOverRoot(false);
        };
        const handleTooltipEnter = () => {
            setIsMouseOverTooltip(true);
        };
        const handleTooltipLeave = () => {
            setIsMouseOverTooltip(false);
        };
        return (React__default.createElement(React__default.Fragment, null,
            children({
                ref: rootRef,
                onMouseEnter: handleEnterRoot,
                onMouseLeave: handleLeaveRoot,
            }),
            tooltipContainer &&
                reactDom.createPortal(React__default.createElement(Root, { className: classNames({
                        [styles.top]: position === 'top',
                        [styles.bottom]: position === 'bottom',
                        [styles.left]: position === 'left',
                        [styles.right]: position === 'right',
                    }), onMouseEnter: handleTooltipEnter, onMouseLeave: handleTooltipLeave, style: {
                        top: tooltipTop,
                        left: tooltipLeft,
                    } }, title), tooltipContainer)));
    }

    const useStyles$i = reactStyleSystem.createStyles(({ css, theme, staticVar }) => ({
        root: css `
    max-height: 90%;
    width: ${theme.block(8)};
    max-width: 100%;
    background-color: ${theme.colors.surface};
    color: ${polished.readableColor(theme.colors.surface)};
    z-index: ${theme.zIndex.modal};
    margin: auto;
    overflow: hidden;
    display: flex;
    flex-direction: column;
    z-index: ${theme.zIndex.modal};
  `,
        container: css `
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    display: flex;
  `,
        backdrop: css `
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    background-color: ${staticVar(polished.transparentize(0.5, 'black'))};
    z-index: ${theme.zIndex.modal};
  `,
    }));
    const Modal = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$i(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'section'), { Root, styles, open, component: _component, 
        // TODO: in the future, we should bind to the escape button for closing
        onClose } = _b, restOfProps = __rest(_b, ["Root", "styles", "open", "component", "onClose"]);
        const [container, setContainer] = React.useState(null);
        // TODO: animations would be a nice-to-have
        React.useEffect(() => {
            if (!open)
                return;
            const container = document.createElement('div');
            container.classList.add(styles.container);
            document.body.appendChild(container);
            setContainer(container);
            return () => {
                document.body.removeChild(container);
                setContainer(null);
            };
        }, [open, props.style, styles.container]);
        React.useEffect(() => {
            if (!container)
                return;
            for (const [k, v] of Object.entries(styles.cssVariableObject)) {
                container.style.setProperty(k, v);
            }
        }, [container, styles.cssVariableObject]);
        // TODO: throw something in the DOM later for SSR SEO
        return (container &&
            reactDom.createPortal(React__default.createElement(React__default.Fragment, null,
                React__default.createElement("div", { className: styles.backdrop, onClick: onClose }),
                React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps))), container));
    });
    Modal.displayName = 'Modal';

    const useStyles$j = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    display: flex;
    flex-direction: column;
    overflow: hidden;
    flex: 0 0 auto;
    padding: ${theme.space(1)} 0;
  `,
    }));
    const ModalHeader = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$j(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'header'), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
        return React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps));
    });
    ModalHeader.displayName = 'ModalHeader';

    const useStyles$k = reactStyleSystem.createStyles(({ css }) => ({
        root: css `
    flex: 1 1 auto;
    position: relative;
    min-height: 50vh;
    overflow: hidden;
    display: flex;
    flex-direction: column;
  `,
    }));
    const ModalContent = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$k(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'div'), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
        return React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps));
    });
    ModalContent.displayName = 'ModalContent';

    const useStyles$l = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    display: flex;
    flex-direction: column;
    overflow: hidden;
    flex: 0 0 auto;
    padding-bottom: ${theme.space(1)};
  `,
    }));
    const ModalFooter = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$l(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'footer'), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
        return React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps));
    });
    ModalFooter.displayName = 'ModalFooter';

    const useStyles$m = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    display: flex;
    justify-content: flex-end;
    padding: 0 ${theme.space(1)};
    margin-top: ${theme.space(1)};
    & > *:not(:first-child) {
      margin-left: ${theme.space(1)};
    }
  `,
    }));
    const ModalActions = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$m(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'div'), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
        return React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps));
    });
    ModalActions.displayName = 'ModalActions';

    const useStyles$n = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    height: 1em;
    line-height: 1;
    width: 1.7em;
    display: inline-flex;
    position: relative;
    overflow: visible;
  `,
        wrapper: css `
    font-size: 1.7em;
    position: absolute;
    top: 50%;
    transform: translateY(-50%);
  `,
    }));
    const Emoji = React.forwardRef((props, ref) => {
        const _a = useStyles$n(props, 'span'), { Root, styles, label, children } = _a, restOfProps = __rest(_a, ["Root", "styles", "label", "children"]);
        return (React__default.createElement(Root, Object.assign({ role: "img", "aria-label": label, ref: ref }, restOfProps),
            React__default.createElement("span", { className: styles.wrapper }, children)));
    });
    Emoji.displayName = 'Emoji';

    const useStyles$o = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    position: absolute;
    width: ${theme.block(2)};
    box-shadow: ${theme.shadows.subtle};
    background-color: ${theme.colors.surface};
    z-index: ${theme.zIndex.modal};
  `,
        top: css `
    transform: translate(-50%, -100%);
  `,
        left: css `
    transform: translate(-100%, -50%);
  `,
        bottom: css `
    transform: translate(-50%, 0);
  `,
        right: css `
    transform: translate(0, -50%);
  `,
        container: css `
    position: fixed;
    width: 100vw;
    height: 100vh;
    top: 0;
    left: 0;
    display: flex;
  `,
        backdrop: css `
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    z-index: ${theme.zIndex.modal};
  `,
    }));
    const Menu = React.forwardRef((props, ref) => {
        const _a = useStyles$o(props, props.component ? props.component : 'div'), { Root, styles, anchorEl, onClose, component, position = 'bottom', children } = _a, restOfProps = __rest(_a, ["Root", "styles", "anchorEl", "onClose", "component", "position", "children"]);
        const [container, setContainer] = React.useState(null);
        const [boundingBox, setBoundingBox] = React.useState({
            top: 0,
            left: 0,
            width: 0,
            height: 0,
        });
        let top;
        let left;
        const topCenter = boundingBox.top + boundingBox.height / 2;
        const leftCenter = boundingBox.left + boundingBox.width / 2;
        // TODO: possibly add other positions:
        // ellipsis menus with icons buttons look bad
        if (position === 'left') {
            top = topCenter;
            left = boundingBox.left;
        }
        else if (position === 'right') {
            top = topCenter;
            left = boundingBox.left + boundingBox.width;
        }
        else if (position === 'bottom') {
            top = boundingBox.top + boundingBox.height;
            left = leftCenter;
        }
        else {
            top = boundingBox.top;
            left = leftCenter;
        }
        // TODO: animations would be nice
        React.useEffect(() => {
            if (!anchorEl)
                return;
            const container = document.createElement('div');
            container.classList.add(styles.container);
            document.body.appendChild(container);
            setContainer(container);
            const { top, left, width, height } = anchorEl.getBoundingClientRect();
            setBoundingBox({ top, left, width, height });
            return () => {
                document.body.removeChild(container);
                setContainer(null);
            };
        }, [anchorEl, styles.container]);
        return (container &&
            reactDom.createPortal(React__default.createElement(React__default.Fragment, null,
                React__default.createElement("div", { className: styles.backdrop, onClick: onClose }),
                React__default.createElement(Root, Object.assign({ className: classNames({
                        [styles.top]: position === 'top',
                        [styles.bottom]: position === 'bottom',
                        [styles.left]: position === 'left',
                        [styles.right]: position === 'right',
                    }), ref: ref, style: { top, left } }, restOfProps), children)), container));
    });
    Menu.displayName = 'Menu';

    function useMediaQuery(query) {
        const normalizedQuery = query.startsWith('@media')
            ? query.substring('@media'.length, query.length)
            : query;
        const mediaQueryList = React.useMemo(() => matchMedia(normalizedQuery), [
            normalizedQuery,
        ]);
        const [matches, setMatches] = React.useState(mediaQueryList.matches);
        React.useEffect(() => {
            const handleChange = (e) => {
                setMatches(e.matches);
            };
            mediaQueryList.addListener(handleChange);
            return () => {
                mediaQueryList.removeListener(handleChange);
            };
        }, [mediaQueryList]);
        return matches;
    }

    const useStyles$p = reactStyleSystem.createStyles(({ css, theme }) => ({
        root: css `
    background-color: ${theme.colors.surface};
    box-shadow: ${theme.shadows.standard};
    width: ${theme.block(3)};
    max-width: 100%;
    height: 100%;
    z-index: ${theme.zIndex.drawer};
  `,
        container: css `
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    display: flex;
  `,
        backdrop: css `
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    background-color: ${polished.transparentize(0.5, 'black')};
    z-index: ${theme.zIndex.drawer};
  `,
    }));
    const Drawer = React.forwardRef((props, ref) => {
        var _a;
        const _b = useStyles$p(props, (_a = props.component) !== null && _a !== void 0 ? _a : 'div'), { Root, styles, component: _component, open, onClose } = _b, restOfProps = __rest(_b, ["Root", "styles", "component", "open", "onClose"]);
        const [container, setContainer] = React.useState(null);
        // TODO: animations would be a nice-to-have
        React.useEffect(() => {
            if (!open)
                return;
            const container = document.createElement('div');
            container.classList.add(styles.container);
            document.body.appendChild(container);
            setContainer(container);
            return () => {
                document.body.removeChild(container);
                setContainer(null);
            };
        }, [open, styles.container]);
        React.useEffect(() => {
            if (!container)
                return;
            for (const [k, v] of Object.entries(styles.cssVariableObject)) {
                container.style.setProperty(k, v);
            }
        }, [container, styles.cssVariableObject]);
        // TODO: SSR/SEO audit
        return (container &&
            reactDom.createPortal(React__default.createElement(React__default.Fragment, null,
                React__default.createElement("div", { className: styles.backdrop, onClick: onClose }),
                React__default.createElement(Root, Object.assign({ ref: ref }, restOfProps))), container));
    });
    Drawer.displayName = 'Drawer';

    exports.Anchor = Anchor;
    exports.Button = Button;
    exports.CheckIcon = CheckIcon;
    exports.Checkbox = Checkbox;
    exports.Chip = Chip;
    exports.ChipThumbnail = ChipThumbnail;
    exports.CircleIcon = CircleIcon;
    exports.Drawer = Drawer;
    exports.Emoji = Emoji;
    exports.FormControl = FormControl;
    exports.HelperText = HelperText;
    exports.Label = Label;
    exports.List = List;
    exports.ListItem = ListItem;
    exports.ListItemButton = ListItemButton;
    exports.Menu = Menu;
    exports.Modal = Modal;
    exports.ModalActions = ModalActions;
    exports.ModalContent = ModalContent;
    exports.ModalFooter = ModalFooter;
    exports.ModalHeader = ModalHeader;
    exports.Radio = Radio;
    exports.RadioGroup = RadioGroup;
    exports.Select = Select;
    exports.Switch = Switch;
    exports.TextArea = TextArea;
    exports.TextInput = TextInput;
    exports.ThemeProvider = ThemeProvider;
    exports.TimesIcon = TimesIcon;
    exports.Tooltip = Tooltip;
    exports.createTheme = createTheme;
    exports.defaultTheme = defaultTheme;
    exports.useMediaQuery = useMediaQuery;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=bundle.umd.js.map
