(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react-style-system'), require('polished')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react-style-system', 'polished'], factory) :
  (global = global || self, factory(global.HackerUiCreateTheme = {}, global.ReactStyleSystem, global.polished));
}(this, (function (exports, reactStyleSystem, polished) { 'use strict';

  const defaultTheme = {
      fonts: {
          button: reactStyleSystem.css `
      font-size: 0.75rem;
      font-weight: bold;
      text-transform: uppercase;
      margin: 0;
    `,
          caption: reactStyleSystem.css `
      font-size: 0.8rem;
      margin: 0;
    `,
          h1: reactStyleSystem.css `
      font-size: 160px;
      font-weight: bold;
      margin: 0;
    `,
          h2: reactStyleSystem.css `
      font-size: 100px;
      font-weight: bold;
      margin: 0;
    `,
          h3: reactStyleSystem.css `
      font-size: 60px;
      font-weight: bold;
      margin: 0;
    `,
          h4: reactStyleSystem.css `
      font-size: 24px;
      font-weight: bold;
      line-height: 1.5;
      margin: 0;
    `,
          h5: reactStyleSystem.css `
      font-size: 16px;
      font-weight: bold;
      line-height: 1.5;
      margin: 0;
    `,
          subtitle1: reactStyleSystem.css `
      font-size: 60px;
      font-weight: bold;
      color: #444;
      margin: 0;
    `,
          subtitle2: reactStyleSystem.css `
      font-size: 24px;
      font-weight: bold;
      color: #444;
      margin: 0;
    `,
          body1: reactStyleSystem.css `
      font-weight: 400;
      text-transform: none;
      font-size: 16px;
      margin: 0;
      line-height: 1.5;
    `,
          body2: reactStyleSystem.css `
      font-weight: 400;
      text-transform: none;
      font-size: 16px;
      margin: 0;
      color: #444;
    `,
      },
      colors: {
          brand: '#000',
          accent: '#2962ff',
          bland: '#ccc',
          danger: '#eb002b',
          warning: '#f56200',
          info: '#2962ff',
          surface: '#fff',
      },
      durations: {
          short: '100ms',
          standard: '250ms',
          long: '500ms',
      },
      shadows: {
          subtle: '0 0 10px 0 rgba(0,0,0,0.10)',
          standard: '0 0 20px 0 rgba(0,0,0,0.13)',
          emphasis: '0 0 30px 0 rgba(0,0,0,0.13)',
      },
      space: n => `${n * 16}px`,
      gap: n => `${n * 48}px`,
      block: n => `${n * 96}px`,
      zIndex: {
          appBar: 1100,
          drawer: 1200,
          modal: 1300,
          notification: 1400,
          tooltip: 1500,
      },
      breakpoints: {
          mobile: '375px',
          tablet: '768px',
          desktop: '1024px',
          desktopLarge: '1440p',
          up: (value) => `@media (min-width: ${polished.math(`${value} + 1px`)})`,
          down: (value) => `@media (max-width: ${value})`,
          between: (min, max) => `@media (max-width: ${min}) and (min-width: ${polished.math(`${max} - 1px`)})`,
      },
  };

  function isObject(maybe) {
      if (!maybe)
          return false;
      return typeof maybe === 'object';
  }
  /**
   * recursively merges a base object with an incoming object
   */
  function recursiveMerge(base, incoming = {}) {
      const keys = [...Object.keys(base), ...Object.keys(incoming)];
      const merged = keys.reduce((merged, key) => {
          const baseValue = base[key];
          const incomingValue = incoming[key];
          const mergedValue = isObject(baseValue)
              ? recursiveMerge(baseValue, incomingValue)
              : (incomingValue || baseValue);
          merged[key] = mergedValue;
          return merged;
      }, {});
      return merged;
  }
  function createTheme(partialTheme = {}) {
      return recursiveMerge(defaultTheme, partialTheme);
  }

  exports.default = createTheme;
  exports.recursiveMerge = recursiveMerge;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=createTheme.js.map
